<?php

namespace Intraservis\AssetManager;

use DirectoryIterator;
use InvalidArgumentException;
use IteratorIterator;
use RecursiveDirectoryIterator;
use RecursiveIteratorIterator;

class AssetManager
{
	private string $baseDir;

	private string $cacheDir;

	private array $types = ["js", "css"];

	private array $assets = [];


	public function __construct(string $baseDir, string $cacheDir)
	{
		$baseDir = rtrim($baseDir, "/");
		$this->baseDir = "$baseDir/";

		$cacheDir = str_replace($baseDir, '', $cacheDir);
		$cacheDir = rtrim($cacheDir, "/");
		$this->cacheDir = "$cacheDir/";
	}


	public function addFile(string $file, string $group = "base"): self
	{
		$type = pathinfo($file, PATHINFO_EXTENSION);

		if ($file[0] !== "/") {
			$file = $this->baseDir . $file;
		}

		if (file_exists($file) === false) {
			throw new InvalidArgumentException("File \"$file\" not found!");
		}

		$mtime = filemtime($file);

		$item = [
			"file" => $file,
			"mtime" => $mtime,
		];

		$key = md5(json_encode($item));

		$this->assets[$type][$group][$key] = $item;

		return $this;
	}


	public function addFiles(array $files, string $group = "base"): self
	{
		foreach ($files as $file) {
			$this->addFile($file, $group);
		}

		return $this;
	}


	public function addGroupsFiles(array $groups): self
	{
		foreach ($groups as $group => $groupFiles) {
			$this->addFiles($groupFiles, $group);
		}

		return $this;
	}


	public function addDir(string $dir, string $group = "base")
	{
		if ($dir[0] !== "/") {
			$dir = $this->baseDir . $dir;
		}

		$options = array_merge([
			"regex" => '/\.(' . implode("|", $this->types) . ')$/i',
			"recursive" => true,
		]);

		if ($options["recursive"] === true) {
			$dirIterator = new RecursiveDirectoryIterator($dir);
			$iterator = new RecursiveIteratorIterator($dirIterator);
		} else {
			$dirIterator = new DirectoryIterator($dir);
			$iterator = new IteratorIterator($dirIterator);
		}

		foreach ($iterator as $fileinfo) {
			$file = $fileinfo->getRealPath();

			if (preg_match($options["regex"], $file) !== 1) {
				continue;
			}

			$type = $fileinfo->getExtension();
			$type = strtolower($type);

			if (in_array($type, $this->types) === false) {
				continue;
			}

			$this->addFile($file, $group);
		}
	}


	public function addDirs(array $dirs, string $group = "base"): self
	{
		foreach ($dirs as $dir) {
			$this->addDir($dir, $group);
		}

		return $this;
	}


	public function addGroupsDirs(array $groups): self
	{
		foreach ($groups as $group => $groupDirs) {
			$this->addDirs($groupDirs, $group);
		}

		return $this;
	}


	public function getAssets(string $type, string $group = null): array
	{
		if ($group !== null) {
			if (array_key_exists($group, $this->assets[$type]) === false) {
				throw new InvalidArgumentException("Group of $type files is not exists!");
			}

			return $this->getGroup($type, $this->assets[$type][$group]);
		}

		$groups = [];
		foreach ($this->assets[$type] as $group => $groupFiles) {
			$groups[$group] = $this->getGroup($type, $groupFiles);
		}

		return $groups;
	}


	private function getGroup(string $type, array $groupFiles): array
	{
		$cacheFile = $this->cacheDir . "$type/" . md5(json_encode($groupFiles)) . ".$type";

		if (file_exists($cacheFile) === false) {
			$content = [];
			foreach ($groupFiles as $groupFile) {
				$fileContent = file_get_contents($groupFile['file']);
				if ($type === "css") {
					$baseUrl = "/" . str_replace($this->baseDir, "", dirname($groupFile["file"])) . "/";
					$fileContent = preg_replace("/url\(['\"](.*?)['\"]\)/", "url(\"" . $baseUrl . "$1\")", $fileContent);
				}
				$content[] = $fileContent;
			}
			file_put_contents($this->baseDir . $cacheFile, implode("\n\n", $content));
		}

		$mtime = filemtime($this->baseDir . $cacheFile);

		return [
			'url' => $cacheFile,
			'mtime' => $mtime,
		];
	}
}
